/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.harness;

import java.awt.AWTException;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Robot;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Set;
import java.util.TreeSet;

import javax.imageio.ImageIO;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;



import org.gfxtest.ImageDiffer.ComparisonResult;
import org.gfxtest.ImageDiffer.ImageComparator;
import org.gfxtest.ImageDiffer.ImageUtils;

/**
 * Main window code and basic application logic for graphic test harness.
 *
 * TODO: dialog for setting reference and tested Java
 * TODO: dialog for setting verbose output
 * TODO: image comparator
 * TODO: image viewer
 * TODO: grid
 * TODO: refactoring!
 *
 * @author Pavel Tisnovsky
 */
public class MainWindow
{
    DefaultListModel testList = new DefaultListModel();
    DefaultTableModel resultTable = new DefaultTableModel(new String[]
    { "Name", "Sample", "Tested", "Result" }, 1);

    private String selectedTestSuite = null;
    private JFrame mainWindow;
    private JLabel testResultsLabel = new JLabel();
    private JTextArea resultTextArea = new JTextArea(5, 40);
    private JButton commandCompile = new JButton("Compile");
    private JButton commandRun = new JButton("Run");
    private JButton commandShowSource = new JButton("Show test source code");
    private JButton commandDeleteClass = new JButton("Delete test class");
    private JButton commandRefresh = new JButton("Refresh test list");
    protected JList jTestList = null;
    protected JTable jResultsTable;
    private JLabel topRightLabel = new JLabel("sample image");
    private JLabel bottomLeftLabel = new JLabel("difference");
    private JLabel bottomRightLabel = new JLabel("test image");
    private Dialogs dialogs = null;
    private Configuration configuration = null;
    private PixelPanel pixelPanel = new PixelPanel();

    private BufferedImage sampleImage = null;
    private BufferedImage testImage = null;
    private BufferedImage diffImage = null;
    private Robot robot = null;

    private boolean tableIsErasing;
    private float scale = 1.0f;

    private JMenuBar createMainMenuBar()
    {
        JMenuBar mainMenu = new JMenuBar();
        mainMenu.add(createMenuFile());
        mainMenu.add(createMenuConfigure());
        mainMenu.add(createMenuView());
        mainMenu.add(createMenuZoom());
        mainMenu.add(createMenuHelp());
        return mainMenu;
    }

    private JMenu createMenuFile()
    {
        JMenu fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);

        JMenuItem fileMenuQuit = new JMenuItem("Quit", KeyEvent.VK_Q);
        fileMenuQuit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
        fileMenuQuit.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                System.exit(0);
            }
        });
        fileMenu.add(fileMenuQuit);
        return fileMenu;
    }

    private JMenu createMenuConfigure()
    {
        JMenu configureMenu = new JMenu("Configure");
        configureMenu.setMnemonic(KeyEvent.VK_C);

        JCheckBoxMenuItem verboseCompilation = new JCheckBoxMenuItem("Verbose compilation", this.configuration.isVerboseCompile());
        verboseCompilation.setHorizontalTextPosition(SwingConstants.RIGHT);
        verboseCompilation.addItemListener(new ItemListener()
        {
            @Override
            public void itemStateChanged(ItemEvent e)
            {
                onCompilationVerboseCheckBoxChanged(e.getStateChange() == ItemEvent.SELECTED);
            }
        });
        configureMenu.add(verboseCompilation);

        JCheckBoxMenuItem verboseTestRun = new JCheckBoxMenuItem("Verbose test run", this.configuration.isVerboseRun());
        verboseTestRun.setHorizontalTextPosition(SwingConstants.RIGHT);
        verboseTestRun.addItemListener(new ItemListener()
        {
            @Override
            public void itemStateChanged(ItemEvent e)
            {
                onTestRunVerboseCheckBoxChanged(e.getStateChange() == ItemEvent.SELECTED);
            }
        });
        configureMenu.add(verboseTestRun);

        configureMenu.addSeparator();

        JMenuItem referenceJava = new JMenuItem("Path to reference JRE");
        referenceJava.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
        referenceJava.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                onSetReferenceJavaMenuItemSelected();
            }
        });
        configureMenu.add(referenceJava);

        JMenuItem testedJava = new JMenuItem("Path to tested JRE");
        testedJava.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK));
        testedJava.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                onSetTestedJavaMenuItemSelected();
            }
        });
        configureMenu.add(testedJava);

        configureMenu.addSeparator();

        JMenuItem loadConfiguration = new JMenuItem("Load configuration");
        loadConfiguration.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                onLoadConfigurationMenuItemSelected();
            }
        });
        configureMenu.add(loadConfiguration);

        JMenuItem saveConfiguration = new JMenuItem("Save configuration");
        saveConfiguration.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                onSaveConfigurationMenuItemSelected();
            }
        });
        configureMenu.add(saveConfiguration);

        return configureMenu;
    }

    private JMenu createMenuView()
    {
        JMenu viewMenu = new JMenu("View");
        viewMenu.setMnemonic(KeyEvent.VK_V);
        return viewMenu;
    }

    private JMenu createMenuZoom()
    {
        JMenu zoomMenu = new JMenu("Zoom");
        zoomMenu.setMnemonic(KeyEvent.VK_Z);

        JMenuItem zoomMenuZoomIn = new JMenuItem("Zoom In", KeyEvent.VK_Z);
        zoomMenuZoomIn.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PLUS, ActionEvent.CTRL_MASK));
        zoomMenuZoomIn.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_EQUALS, ActionEvent.CTRL_MASK));
        zoomMenuZoomIn.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                onZoomInSelected();
            }
        });
        zoomMenu.add(zoomMenuZoomIn);

        JMenuItem zoomMenuZoomOut = new JMenuItem("Zoom Out", KeyEvent.VK_O);
        zoomMenuZoomOut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_MINUS, ActionEvent.CTRL_MASK));
        zoomMenuZoomOut.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                onZoomOutSelected();
            }
        });
        zoomMenu.add(zoomMenuZoomOut);

        JMenuItem zoomMenuReset = new JMenuItem("Normal Size", KeyEvent.VK_N);
        zoomMenuReset.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_0, ActionEvent.CTRL_MASK));
        zoomMenuReset.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                onResetZoomMenuSelected();
            }
        });
        zoomMenu.add(zoomMenuReset);

        return zoomMenu;
    }

    private JMenu createMenuHelp()
    {
        JMenu helpMenu = new JMenu("Help");
        helpMenu.setMnemonic(KeyEvent.VK_H);

        JMenuItem menuHelpAbout = new JMenuItem("About", KeyEvent.VK_A);
        menuHelpAbout.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
        menuHelpAbout.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                onHelpAboutMenuSelected();
            }
        });
        helpMenu.add(menuHelpAbout);

        return helpMenu;
    }

    private GridBagConstraints constraints(int gridx, int gridy, int gridWidth, int gridHeight, double weightx, double weighty, int fill)
    {
        Insets insets = new Insets(2, 2, 2, 2);
        return new GridBagConstraints(gridx, gridy, gridWidth, gridHeight, weightx, weighty, GridBagConstraints.CENTER, fill, insets, 0, 0);        
    }

    private void createTopLeftPanelControls(JPanel panel)
    {
        this.jTestList = new JList(this.testList);
        this.jTestList.setVisibleRowCount(20);
        this.jTestList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        this.jTestList.addListSelectionListener(new ListSelectionListener()
        {
            @Override
            public void valueChanged(ListSelectionEvent e)
            {
                if (!e.getValueIsAdjusting())
                {
                    int selectedIndex = MainWindow.this.jTestList.getSelectedIndex();
                    if (selectedIndex != -1)
                    {
                        selectTestSuite(selectedIndex);
                    }
                }
            }
        });
        JScrollPane testListPane = new JScrollPane(this.jTestList);

        this.jResultsTable = new JTable(this.resultTable);
        this.jResultsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        this.jResultsTable.getSelectionModel().addListSelectionListener(new ListSelectionListener()
        {
            @Override
            public void valueChanged(ListSelectionEvent e)
            {
                if (!e.getValueIsAdjusting())
                {
                    int selectedIndex = MainWindow.this.jResultsTable.getSelectedRow();
                    if (selectedIndex != -1)
                    {
                        selectTestResult(selectedIndex);
                    }
                }
            }
        });
        JScrollPane resultsPane = new JScrollPane(this.jResultsTable);

        this.resultTextArea.setEditable(false);
        JScrollPane resultTextPane = new JScrollPane(this.resultTextArea);

        this.commandCompile.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                onCompileButtonPressed();
            }
        });

        this.commandRun.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                onButtonRunPressed();
            }
        });

        this.commandDeleteClass.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e) {
                onButtonDeleteClassPressed();
            }
        });

        this.commandShowSource.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e) {
                onButtonShowSourcePressed();
            }
        });

        this.commandRefresh.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                onRefreshTestListPressed();
            }
        });

        this.resetTestResultsLabel();
        this.testResultsLabel.setOpaque(true);
        this.testResultsLabel.setPreferredSize(new Dimension(200, 20));
        this.testResultsLabel.setMinimumSize(new Dimension(200, 20));

        panel.add(new JLabel("Gfx. test"),    constraints(0, 0, 1, 1, 0.4, 0.0, GridBagConstraints.HORIZONTAL));
        panel.add(new JLabel("Commands"),     constraints(1, 0, 1, 1, 0.3, 0.0, GridBagConstraints.HORIZONTAL));
        panel.add(this.testResultsLabel,      constraints(2, 0, 1, 1, 0.3, 0.0, GridBagConstraints.HORIZONTAL));
        panel.add(testListPane,               constraints(0, 1, 1, 7, 0.4, 1.0, GridBagConstraints.BOTH));
        panel.add(resultsPane,                constraints(2, 1, 1, 6, 0.3, 0.5, GridBagConstraints.BOTH));
        panel.add(resultTextPane,             constraints(1, 7, 2, 1, 0.3, 0.5, GridBagConstraints.BOTH));

        panel.add(this.commandShowSource,     constraints(1, 1, 1, 1, 0.1, 0.0, GridBagConstraints.HORIZONTAL));
        panel.add(this.commandCompile,        constraints(1, 2, 1, 1, 0.1, 0.0, GridBagConstraints.HORIZONTAL));
        panel.add(this.commandRun,            constraints(1, 3, 1, 1, 0.1, 0.0, GridBagConstraints.HORIZONTAL));
        panel.add(this.commandDeleteClass,    constraints(1, 4, 1, 1, 0.1, 0.0, GridBagConstraints.HORIZONTAL));
        panel.add(this.pixelPanel,            constraints(1, 5, 1, 1, 0.1, 1.0, GridBagConstraints.BOTH));
        panel.add(this.commandRefresh,        constraints(1, 6, 1, 1, 0.1, 0.0, GridBagConstraints.HORIZONTAL));
    }

    /**
     * Create and set up the main window.
     */
    protected void createMainWindow()
    {
        try
        {
            this.robot = new Robot();
        }
        catch (AWTException e)
        {
            e.printStackTrace();
        }
        JFrame window = new JFrame(Configuration.getWindowTitle());
        this.dialogs = new Dialogs(Configuration.getWindowTitle(), window);
        window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        window.setJMenuBar(createMainMenuBar());

        JSplitPane horizontalSplitPane = createHorizontalSplitPane();
        JSplitPane leftVerticalSplitPane = createRightVerticalSplitPane();
        JSplitPane rightVerticalSplitPane = createRightVerticalSplitPane();

        horizontalSplitPane.add(leftVerticalSplitPane);
        horizontalSplitPane.add(rightVerticalSplitPane);

        leftVerticalSplitPane.add(createTopLeftPanel());
        rightVerticalSplitPane.add(createTopRightPanel());
        leftVerticalSplitPane.add(createBottomLeftPanel());
        rightVerticalSplitPane.add(createBottomRightPanel());

        window.getContentPane().add(horizontalSplitPane);

        // Display the window.
        window.setSize(this.configuration.getMainWindowWidth(), this.configuration.getMainWindowHeight());
        window.setVisible(true);
        this.setMainWindow(window);
    }

    private JSplitPane createHorizontalSplitPane()
    {
        JSplitPane horizontalSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        horizontalSplitPane.setOneTouchExpandable(true);
        horizontalSplitPane.setDividerLocation(1. / 2);
        return horizontalSplitPane;
    }

    private JSplitPane createRightVerticalSplitPane()
    {
        JSplitPane rightVerticalSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        rightVerticalSplitPane.setOneTouchExpandable(true);
        rightVerticalSplitPane.setDividerLocation(1. / 2);
        return rightVerticalSplitPane;
    }

    private JPanel createTopLeftPanel()
    {
        JPanel topLeftPanel = new JPanel();
        topLeftPanel.setLayout(new GridBagLayout());
        //topLeftPanel.setMinimumSize(new Dimension(320, 240));
        createTopLeftPanelControls(topLeftPanel);
        return topLeftPanel;
    }

    private JPanel createTopRightPanel()
    {
        JPanel topRightPanel = new JPanel();
        topRightPanel.setMinimumSize(new Dimension(320, 240));
        topRightPanel.add(this.topRightLabel);
        topRightPanel.addMouseWheelListener(new MouseWheelListener()
        {
            @Override
            public void mouseWheelMoved(MouseWheelEvent e)
            {
                onMouseWheelRotated(e);
            }
        });
        topRightPanel.addMouseMotionListener(new MouseMotionListener()
        {
            @Override
            public void mouseDragged(MouseEvent e)
            {
                // correct to be empty
            }
            
            @Override
            public void mouseMoved(MouseEvent e)
            {
                onMouseMovedOverSampleImage(e);
            }
        });
        return topRightPanel;
    }

    private JPanel createBottomLeftPanel()
    {
        JPanel bottomLeftPanel = new JPanel();
        bottomLeftPanel.setMinimumSize(new Dimension(320, 240));
        bottomLeftPanel.add(this.bottomLeftLabel);
        bottomLeftPanel.addMouseWheelListener(new MouseWheelListener()
        {
            @Override
            public void mouseWheelMoved(MouseWheelEvent e)
            {
                onMouseWheelRotated(e);
            }
        });
        return bottomLeftPanel;
    }

    private JPanel createBottomRightPanel()
    {
        JPanel bottomRightPanel = new JPanel();
        bottomRightPanel.setMinimumSize(new Dimension(320, 240));
        bottomRightPanel.add(this.bottomRightLabel);
        bottomRightPanel.addMouseWheelListener(new MouseWheelListener()
        {
            @Override
            public void mouseWheelMoved(MouseWheelEvent e)
            {
                onMouseWheelRotated(e);
            }
        });
        bottomRightPanel.addMouseMotionListener(new MouseMotionListener()
        {
            @Override
            public void mouseDragged(MouseEvent e)
            {
                // correct to be empty
            }
            
            @Override
            public void mouseMoved(MouseEvent e)
            {
                onMouseMovedOverTestImage(e);
            }
        });
        return bottomRightPanel;
    }

    /**
     * @param mainWindow the mainWindow to set
     */
    public void setMainWindow(JFrame mainWindow)
    {
        this.mainWindow = mainWindow;
    }

    /**
     * @return the mainWindow
     */
    public JFrame getMainWindow()
    {
        return this.mainWindow;
    }

    protected void resetTestResultsLabel()
    {
        this.testResultsLabel.setText("no test selected");
        this.commandCompile.setEnabled(false);
        this.commandRun.setEnabled(false);
        this.commandDeleteClass.setEnabled(false);
        this.commandShowSource.setEnabled(false);
    }

    private void setTestResultsLabel(String selectedTest)
    {
        this.testResultsLabel.setText(selectedTest + " results");
        this.commandCompile.setEnabled(true);
        this.commandRun.setEnabled(true);
        this.commandDeleteClass.setEnabled(true);
        this.commandShowSource.setEnabled(true);
    }

    protected void selectTestSuite(int selectedIndex)
    {
        if (selectedIndex != -1 && this.testList.getSize() > selectedIndex)
        {
            eraseResultTable();
            this.selectedTestSuite = String.valueOf(this.testList.get(selectedIndex));
            /*
            JOptionPane.showMessageDialog(this.mainWindow,
                    "Selected test: " + this.selectedTest, WINDOW_TITLE,
                    JOptionPane.INFORMATION_MESSAGE);
                    */
            setTestResultsLabel(this.selectedTestSuite);
            refreshTestResults(this.selectedTestSuite);
        }
    }

    protected void selectTestResult(int selectedIndex)
    {
        if (selectedIndex != -1 && this.resultTable.getRowCount() > selectedIndex && this.selectedTestSuite != null
                && !this.tableIsErasing)
        {
            String testName = this.selectedTestSuite + "_" + this.resultTable.getValueAt(selectedIndex, 0).toString()
                    + ".png";
            String sampleFileName = FileSystemTools.constructTestDirectoryName(this.configuration.getOutputDirectory(), this.selectedTestSuite);
            String testFileName = FileSystemTools.constructTestDirectoryName(this.configuration.getOutputDirectory(), this.selectedTestSuite);
            loadAndShowSampleImage(testName, sampleFileName);
            loadAndShowTestImage(testName, testFileName);
            computeAndShowImagesDifference(testName);
            /*
             * JOptionPane.showMessageDialog(this.mainWindow, "Selected test: "
             * + testName, WINDOW_TITLE, JOptionPane.INFORMATION_MESSAGE);
             */
        }
    }

    private void loadAndShowTestImage(String testName, String testFileName)
    {
        this.testImage = null;
        File testImageFile = new File(testFileName, testName);
        if (testImageFile.exists() && testImageFile.isFile())
        {
            try
            {
                this.testImage = ImageIO.read(new File(testFileName, testName));
                this.bottomRightLabel.setIcon(new ImageIcon(this.testImage));
                this.bottomRightLabel.setText("");
            }
            catch (IOException e)
            {
                e.printStackTrace();
                this.bottomRightLabel.setText(e.getMessage());
            }
        }
    }

    private void loadAndShowSampleImage(String testName, String sampleFileName)
    {
        this.sampleImage = null;
        File sampleImageFile = new File(sampleFileName, testName);
        if (sampleImageFile.exists() && sampleImageFile.isFile())
        {
            try
            {
                this.sampleImage = ImageIO.read(new File(sampleFileName, testName));
                
                this.topRightLabel.setIcon(new ImageIcon(this.sampleImage));
                this.topRightLabel.setText("");
            }
            catch (IOException e)
            {
                e.printStackTrace();
                this.topRightLabel.setText(e.getMessage());
            }
        }
    }

    private void computeAndShowImagesDifference(String testName)
    {
        if (this.testImage != null && this.sampleImage != null)
        {
            ImageComparator comparator = new ImageComparator();
            org.gfxtest.ImageDiffer.Configuration cfg = new org.gfxtest.ImageDiffer.Configuration();
            ComparisonResult result = comparator.diffImages(testName, new BufferedImage[] { this.testImage, this.sampleImage }, cfg);
            this.diffImage = result.getDiffImage();
            this.bottomLeftLabel.setIcon(new ImageIcon(this.diffImage));
            this.bottomLeftLabel.setText("");
        }
        else
        {
            this.bottomLeftLabel.setText("difference");
        }
    }

    protected void refreshTestList()
    {
        this.selectedTestSuite = null;
        File dir = new File(System.getProperty("user.dir"), Paths.PATH_TO_TESTSUITES);
        Set<String> testNames = new TreeSet<String>();
        for (String name : dir.list())
        {
            testNames.add(name.substring(0, name.indexOf('.')));
        }
        this.testList.clear();
        for (String name : testNames)
        {
            this.testList.addElement(name);
        }
        this.resetTestResultsLabel();
        eraseResultTable();
    }

    private String constructTestNameFromFileName(String fileName)
    {
        // special cases - it should not happen!
        if (fileName.indexOf('_') == -1 || fileName.indexOf('.') == -1)
        {
            return fileName;
        }
        String withoutPrefix = fileName.substring(fileName.indexOf('_') + 1);
        String withoutSuffix = withoutPrefix.substring(0, withoutPrefix.indexOf('.'));
        return withoutSuffix;
    }

    private Set<String> getResultList(String directory, String selectedTestName)
    {
        File testDirectory = new File(directory, selectedTestName);
        Set<String> results = new TreeSet<String>();
        if (testDirectory.exists() && testDirectory.isDirectory())
        {
            for (String fileName : testDirectory.list())
            {
                String testName = constructTestNameFromFileName(fileName);
                results.add(testName);
            }
        }
        return results;
    }

    private void refreshTestResults(String selectedTestName)
    {
        eraseResultTable();
        Set<String> samples = getResultList(this.configuration.getSamplesDirectory(), selectedTestName);
        Set<String> output = getResultList(this.configuration.getOutputDirectory(), selectedTestName);
        Set<String> allResults = new TreeSet<String>();
        allResults.addAll(samples);
        allResults.addAll(output);
        for (String testName : allResults)
        {
            String sample = samples.contains(testName) ? "ok" : "x";
            String tested = output.contains(testName) ? "ok" : "x";
            String result = "?";
            this.resultTable.addRow(new Object[] { testName, sample, tested, result });
        }
    }

    private void eraseResultTable()
    {
        this.tableIsErasing = true;
        while (this.resultTable.getRowCount() != 0)
        {
            this.resultTable.removeRow(0);
        }
        this.tableIsErasing = false;
    }

    protected void onMouseMovedOverSampleImage(MouseEvent e)
    {
        if (this.sampleImage != null && this.robot != null)
        {
            Point location = e.getLocationOnScreen();
            Color c1 = this.robot.getPixelColor(location.x, location.y);
            int offsetY = this.bottomRightLabel.getY() - this.topRightLabel.getY();
            Color c2 = this.robot.getPixelColor(location.x, location.y + offsetY);
            this.pixelPanel.showPixelValues(c1, c2);
        }
    }

    protected void onMouseMovedOverTestImage(MouseEvent e)
    {
        if (this.testImage != null && this.robot != null)
        {
            Point location = e.getLocationOnScreen();
            Color c1 = this.robot.getPixelColor(location.x, location.y);
            int offsetY = this.bottomRightLabel.getY() - this.topRightLabel.getY();
            Color c2 = this.robot.getPixelColor(location.x, location.y - offsetY);
            this.pixelPanel.showPixelValues(c1, c2);
        }
    }

    protected void onMouseWheelRotated(MouseWheelEvent e)
    {
        int rotation = e.getWheelRotation();
        if (rotation < 0)
        {
            this.scale *= 1.2;
            redrawImages();
        }
        else if (rotation > 0)
        {
            this.scale /= 1.2;
            redrawImages();
        }
    }

    private void redrawImages()
    {
        this.topRightLabel.setIcon(new ImageIcon(ImageUtils.scaleImage(this.sampleImage, this.scale)));
        this.bottomRightLabel.setIcon(new ImageIcon(ImageUtils.scaleImage(this.testImage, this.scale)));
        this.bottomLeftLabel.setIcon(new ImageIcon(ImageUtils.scaleImage(this.diffImage, this.scale)));
    }

    protected void onCompilationVerboseCheckBoxChanged(boolean verboseCompile)
    {
        this.configuration.setVerboseCompile(verboseCompile);
    }

    protected void onTestRunVerboseCheckBoxChanged(boolean verboseRun)
    {
        this.configuration.setVerboseRun(verboseRun);
    }

    private String selectPathToJava(String oldPath, String message)
    {
        JFileChooser fc = new JFileChooser(new File(oldPath));
        fc.setDialogTitle(message);
        if (fc.showOpenDialog(this.mainWindow) == JFileChooser.APPROVE_OPTION)
        {
            File file = fc.getSelectedFile();
            System.out.println("selected path: " + file.toString());
            return file.getAbsolutePath();
        }
        return null;
    }

    protected void onSetReferenceJavaMenuItemSelected()
    {
        String selectedPath = selectPathToJava(this.configuration.getReferenceJava(),
                "Path to reference JRE (to file java)");
        if (selectedPath != null)
        {
            this.configuration.setReferenceJava(selectedPath);
        }
    }

    protected void onSetTestedJavaMenuItemSelected()
    {
        String selectedPath = selectPathToJava(this.configuration.getTestedJava(), "Path to tested JRE (to file java)");
        if (selectedPath != null)
        {
            this.configuration.setTestedJava(selectedPath);
        }
    }

    protected void onLoadConfigurationMenuItemSelected()
    {
        this.dialogs.showDialogAfterLoadConfiguration(this.configuration.readConfiguration());
    }

    protected void onSaveConfigurationMenuItemSelected()
    {
        this.dialogs.showDialogAfterSaveConfiguration(this.configuration.writeConfiguration());
    }

    protected void onHelpAboutMenuSelected()
    {
        this.dialogs.showAboutDialog();
    }

    protected void onResetZoomMenuSelected()
    {
        this.scale = 1.0f;
        redrawImages();
    }

    protected void onZoomInSelected()
    {
        this.scale *= 1.2f;
        redrawImages();
    }

    protected void onZoomOutSelected()
    {
        this.scale /= 1.2f;
        redrawImages();
    }

    protected void onCompileButtonPressed()
    {
        if (this.selectedTestSuite == null)
        {
            this.dialogs.showNoTestSelectedMessage();
            return;
        }
        try
        {
            String output = ExternalCommands.compileSelectedTestSuite(this.selectedTestSuite, this.configuration.isVerboseCompile());
            this.resultTextArea.setText(output);
        }
        catch (IOException e)
        {
            this.dialogs.showDialogWhenCompilationFailed(e);
        }
        catch (InterruptedException e)
        {
            this.dialogs.showDialogWhenCompilationFailed(e);
        }
        catch (RuntimeException e)
        {
            this.dialogs.showDialogWhenCompilationFailed(e);
        }
    }

    protected void onButtonRunPressed()
    {
        if (this.selectedTestSuite == null)
        {
            this.dialogs.showNoTestSelectedMessage();
            return;
        }
        try
        {
            String output = ExternalCommands.runTestOnGivenJDK(this.configuration.getReferenceJava(), this.selectedTestSuite, this.configuration.getSamplesDirectory(), this.configuration.isVerboseRun());
            this.resultTextArea.setText(output);
            output = ExternalCommands.runTestOnGivenJDK(this.configuration.getTestedJava(), this.selectedTestSuite, this.configuration.getOutputDirectory(), this.configuration.isVerboseRun());
            this.resultTextArea.append(output);
        }
        catch (IllegalThreadStateException e)
        {
            this.dialogs.showDialogWhenRunTestFailed(e);
        }
        catch (IOException e)
        {
            this.dialogs.showDialogWhenRunTestFailed(e);
        }
        catch (InterruptedException e)
        {
            this.dialogs.showDialogWhenRunTestFailed(e);
        }
        catch (RuntimeException e)
        {
            this.dialogs.showDialogWhenRunTestFailed(e);
        }
        refreshTestResults(this.selectedTestSuite);
    }

    protected void onButtonShowSourcePressed()
    {
        if (this.selectedTestSuite == null)
        {
            this.dialogs.showNoTestSelectedMessage();
            return;
        }
        new SourceViewer().showSource(this.selectedTestSuite);
    }

    protected void onButtonDeleteClassPressed()
    {
        if (this.selectedTestSuite == null)
        {
            this.dialogs.showNoTestSelectedMessage();
        }
        String outputDirectory = Configuration.getTestClassesDirectory();
        File fileToDelete = new File(outputDirectory, this.selectedTestSuite + ".class");
        if (!fileToDelete.exists())
        {
            try
            {
                this.dialogs.showWarningMessage("File " + fileToDelete.getCanonicalPath() + " does not exists");
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
        else
        {
            if (fileToDelete.delete())
            {
                try
                {
                    this.dialogs.showInfoMessage("File " + fileToDelete.getCanonicalPath() + " deleted");
                }
                catch (IOException e) {
                    e.printStackTrace();
                }
            }
            else
            {
                try
                {
                    this.dialogs.showErrorMessage("File " + fileToDelete.getCanonicalPath() + " can not be deleted");
                }
                catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    protected void onRefreshTestListPressed()
    {
        refreshTestList();
    }

    private void runTestHarness()
    {
        this.configuration = new Configuration();
        this.configuration.readConfiguration();

        refreshTestList();

        // Schedule a job for the event-dispatching thread:
        // creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                createMainWindow();
            }
        });
    }

    /**
     * @param args
     */
    public static void main(String[] args)
    {
        new MainWindow().runTestHarness();
    }

}
